"""earlyster_probability - Binomial and Gaussian Distribution Probability Library"""

__version__ = '0.1.1'
__author__ = 'Justin Early <earlyster@gmail.com>'
__all__ = []

from .gaussian_distribution import Gaussian
from .binomial_distribution import Binomial



"""Binomial Distribution Module
Contains Binomial Class
"""
import math
import matplotlib.pyplot as plt
from .general_distribution import Distribution

class Binomial(Distribution):
    """ Binomial distribution class for calculating and
    visualizing a Binomial distribution.

    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the
        data file
        p (float) representing the probability of an event occurring
        n (int) number of trials
    """
    def __init__(self, prob=.5, size=20):
        self.number_of_trials = size
        self.probability_of_event = prob
        Distribution.__init__(self, self.calculate_mean(),
            self.calculate_stdev())

    def calculate_mean(self):
        """Function to calculate the mean from p and n

        Args:
            None

        Returns:
            float: mean of the data set

        """
        self.mean = self.probability_of_event * self.number_of_trials
        return self.mean



    def calculate_stdev(self):
        """Function to calculate the standard deviation from p and n.

        Args:
            None

        Returns:
            float: standard deviation of the data set
        """

        self.stdev = math.sqrt(self.number_of_trials * \
            self.probability_of_event * (1 - self.probability_of_event))

        return self.stdev

    def replace_stats_with_data(self):
        """Function to calculate p and n from the data set

        Args:
            None

        Returns:
            float: the p value
            float: the n value

        """

        self.number_of_trials = len(self.data)
        self.probability_of_event = 1.0 * sum(self.data) / len(self.data)
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()

        return self.probability_of_event, self.number_of_trials

    def plot_bar(self):
        """Function to output a histogram of the instance variable data using
        matplotlib pyplot library.

        Args:
            None

        Returns:
            None
        """

        plt.bar(x = ['0', '1'], height = [(1 - self.probability_of_event) \
            * self.number_of_trials, self.probability_of_event \
                * self.number_of_trials])
        plt.title('Bar Chart of Data')
        plt.xlabel('outcome')
        plt.ylabel('count')

    def pdf(self, k):
        """Probability density function calculator for the gaussian distribution.

        Args:
            x (float): point for calculating the probability density function


        Returns:
            float: probability density function output
        """

        a_number = math.factorial(self.number_of_trials) / (math.factorial(k) \
            * (math.factorial(self.number_of_trials - k)))
        b_number = (self.probability_of_event ** k) \
            * (1 - self.probability_of_event) ** (self.number_of_trials - k)
        return a_number * b_number

    def plot_bar_pdf(self):
        """Function to plot the pdf of the binomial distribution

        Args:
            None

        Returns:
            list: x_number values for the pdf plot
            list: y_number values for the pdf plot

        """

        x_number = []
        y_number = []

        # calculate the x values to visualize
        for i in range(self.number_of_trials + 1):
            x_number.append(i)
            y_number.append(self.pdf(i))

        # make the plots
        plt.bar(x_number, y_number)
        plt.title('Distribution of Outcomes')
        plt.ylabel('Probability')
        plt.xlabel('Outcome')

        plt.show()

        return x_number, y_number

    def __add__(self, other):
        """Function to add together two Binomial distributions with equal p

        Args:
            other (Binomial): Binomial instance

        Returns:
            Binomial: Binomial distribution

        """

        try:
            assert self.probability_of_event == other.probability_of_event, 'p values are not equal'
        except AssertionError as error:
            print("Assertion Error. {}".format(error))
            raise

        result = Binomial()
        result.number_of_trials = self.number_of_trials + other.number_of_trials
        result.probability_of_event = self.probability_of_event
        result.calculate_mean()
        result.calculate_stdev()

        return result

    def __repr__(self):
        """Function to output the characteristics of the Binomial instance

        Args:
            None

        Returns:
            string: characteristics of the Gaussian

        """

        return "mean {}, standard deviation {}, p {}, n {}".\
        format(self.mean, self.stdev, self.probability_of_event, \
            self.number_of_trials)



'''Gaussian Distribution module'''

import math
import warnings
import matplotlib.pyplot as plt
from .general_distribution import Distribution

class Gaussian(Distribution):

    """ Gaussian distribution class for calculating and
    visualizing a Gaussian distribution.

    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats extracted from the data file

    """

    def calculate_mean(self):
        """Method to calculate the mean of the data set.

        Args:
            None

        Returns:
            float: mean of the data set
        """
        avg = 1.0 * sum(self.data) / len(self.data)
        self.mean = avg
        return self.mean


    def calculate_stdev(self, sample=True):

        """Method to calculate the standard deviation of the data set.

        Args:
            sample (bool): whether the data represents a sample or population

        Returns:
            float: standard deviation of the data set

        """

        if sample:
            n_number = len(self.data) - 1
        else:
            n_number = len(self.data)
        mean = self.mean
        sigma = 0

        for data in self.data:
            sigma += (data-mean) ** 2

        sigma = math.sqrt(sigma / n_number)

        self.stdev = sigma

        return self.stdev

    def read_data_file_and_calc(self, file_path, sample=True):
        """Method to read in data from a txt file. The txt file should have
        one number (float) per line. The numbers are stored in the data
        attribute.

        After reading in the file, the mean and standard deviation are
        calculated

        Args:
            file_path (string): path to the file to read for data,
                must have read access.
            sample (bool): whether the data represents a sample or population

        Returns:
            None

        """

        # This code opens a data file and appends the data to a list called data_list
        super().read_data_file(file_path)

        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev(sample)

    def plot_histogram(self):
        """Method to output a histogram of the instance variable data using
        matplotlib pyplot library.

        Args:
            None

        Returns:
            None
        """

        plt.hist(self.data)
        plt.title("Histogram of Data")
        plt.xlabel("Data")
        plt.ylabel("Count")
        plt.show()

    def pdf(self, point):
        """Probability density function calculator for the gaussian
        distribution.

        Args:
            point (float): point for calculating the probability density function


        Returns:
            float: probability density function output
        """
        return (1.0 \
        / (self.stdev * math.sqrt(2*math.pi))) * \
            math.exp(-0.5*((point - self.mean)/self.stdev)**2)

    def plot_histogram_pdf(self, n_spaces = 50):

        """Method to plot the normalized histogram of the data and a plot of the
        probability density function along the same range

        Args:
            n_spaces (int): number of data points

        Returns:
            list: x values for the pdf plot
            list: y values for the pdf plot

        """

        min_range = min(self.data)
        max_range = max(self.data)

         # calculates the interval between x values
        interval = 1.0 * (max_range - min_range) / n_spaces

        x_number = []
        y_number = []

        # calculate the x values to visualize
        for i in range(n_spaces):
            tmp = min_range + interval*i
            x_number.append(tmp)
            y_number.append(self.pdf(tmp))

        # make the plots
        fig, axes = plt.subplots(2,sharex=True)
        fig.subplots_adjust(hspace=.5)
        axes[0].hist(self.data, density=True)
        axes[0].set_title('Normed Histogram of Data')
        axes[0].set_ylabel('Density')

        axes[1].plot(x_number, y_number)
        axes[1].set_title('Normal Distribution for \n Sample Mean and Sample Standard Deviation')
        axes[0].set_ylabel('Density')
        plt.show()

        return x_number, y_number

    def __add__(self, other):
        """
        Funcaiton to add together two Gaussian distributions

        Args:
            other (Gaussian): Gaussian instance

        Returns:
            Guassian: Gaussian distribution
        """
        result = Gaussian()
        result.mean = self.mean + other.mean
        result.stdev = math.sqrt(self.stdev ** 2 + other.stdev ** 2)

        return result

    def __repr__(self):
        """
        Function to output the characteristics of the Gaussian instance

        Args:
            None

        Returns:
            sting: characteristics of the Gaussian
        """
        return "mean {}, standard deviation {}".format(self.mean, self.stdev)

    def __getattr__(self, name):
        ''' will only get called for undefined attributes '''
        warnings.warn('No member "%s" contained in settings config.' % name)
        return ''
